@@ -9,13 +9,13 @@ MODULE wrf_constants
     REAL(KIND=8), PARAMETER :: PI = 3.1415926535897932384626433D0
     REAL(KIND=8), PARAMETER :: RAD_PER_DEG = PI/180.D0
     REAL(KIND=8), PARAMETER :: DEG_PER_RAD = 180.D0/PI
-    REAL(KIND=8), PARAMETER :: DEFAULT_FILL = 9.9692099683868690E36
+    REAL(KIND=8), PARAMETER :: DEFAULT_FILL = 9.9692099683868690D36
     INTEGER(KIND=1), PARAMETER :: DEFAULT_FILL_INT8 = -127
     INTEGER(KIND=2), PARAMETER :: DEFAULT_FILL_INT16 = -32767
     INTEGER(KIND=4), PARAMETER :: DEFAULT_FILL_INT32 = -2147483647
     INTEGER(KIND=8), PARAMETER :: DEFAULT_FILL_INT64 = INT(-9223372036854775806D0, KIND=8)
     REAL(KIND=4), PARAMETER :: DEFAULT_FILL_FLOAT = 9.9692099683868690E36
-    REAL(KIND=8), PARAMETER :: DEFAULT_FILL_DOUBLE = 9.9692099683868690E36
+    REAL(KIND=8), PARAMETER :: DEFAULT_FILL_DOUBLE = 9.9692099683868690D36
     CHARACTER(LEN=1), PARAMETER :: DEFAULT_FILL_CHAR = ACHAR(0)
 
 
