@@ -1,5 +1,4 @@
-from __future__ import (absolute_import, division, print_function, 
-                        unicode_literals)
+from __future__ import (absolute_import, division, print_function)
 
 import numpy as np
 
